function RF_full = GetFullRF( RF_sparse, T )

    % Convert sparse RF waveforms (samples only where RF is supposed to be 
    % played during the excitation) to full RF waveforms (one sample per 
    % time sample). 

    %check correct size
    if numel( RF_sparse ) ~= T.Pulse.MatSize
       error('Sparse RF input must match matrix size (numel(RF) = %i, Matrix = %i columns)',numel(RF_sparse),T.Pulse.MatSize) 
    end

    %allocate    
    RF_full     = nan( T.Pulse.RFfullLength, 1 ) + 1j * nan( T.Pulse.RFfullLength, 1 );
 
    %set nonzero entries
    NonZero_idx = T.Pulse.RFfull_idx( T.Pulse.RFfull_idx ~= 0 );
    RF_full( T.Pulse.RFfull_NonZeros == 1 ) 	= RF_sparse( unique( NonZero_idx ) );
    
    RF_full( isnan( RF_full ) ) = 0.0;
    
end