function J = EvalObjective3D_Jac( x, AuxData )    

    %#codegen

    %{ 
    Method:
    -------
    Evaluates the objective function (equivalent to the pulse duration)
    based on the vector x to be optimized (x constains the variable 
    gradient magnitudes to be optimized) and the AuxData struct. AuxData
    contains information about the constraint on the control points,
    several options such as the penatly function behaviour, etc. In the
    method the gradient are solved within two subsequent control points
    based on the current gradient values at the control points. In addition
    the Jacobian and the Hessian are stored and formated accordingly to the
    constraints (using the derivative matrix AuxData.D)
    
    Input:
    ------
    
    x       - vector of gradient magnitudes
    AuxData - struct (cf. SetUpAuxData3D)

    Output:
    -------
    
    J   - Jacobian at x
    
    %}

    NCP = size(AuxData.R,1);

    %******************************************************************
    %   C O M P U T E   O B J E C T I V E   A N D   J A C O B I A N
    %******************************************************************  
    %fprintf('Evaluating (%i): Objective, Jacobian\n',nargout)

    %back out full gradient vector
    G_full  = AuxData.D' * x;
    Jt      = zeros( 3*NCP, 1 );

    for k = 1 : NCP-1

        %evaluate objective function using the previously defined gradient
        %vectors, depending on the number of variables to be returned, the 
        % associated method is used
        [~, gradIn, gradOut] = FastTimeOptGrad_3D_deriv1( G_full( 3*k-2 ), G_full( 3*k+1 ), G_full( 3*k-1 ) , G_full( 3*k+2 ), G_full( 3*k ), G_full( 3*k+3 ), ...
                                                          AuxData.dkx(k), AuxData.dky(k), AuxData.dkz(k), ...
                                                          AuxData.Gmax, AuxData.Smax, AuxData.OptsVector );

        %set Jacobian
        Jt(3*(k-1)+1)  = Jt(3*(k-1)+1) + gradIn(1);   Jt(3*k+1)  = Jt(3*k+1) + gradOut(1);     
        Jt(3*(k-1)+2)  = Jt(3*(k-1)+2) + gradIn(2);   Jt(3*k+2)  = Jt(3*k+2) + gradOut(2);   
        Jt(3*(k-1)+3)  = Jt(3*(k-1)+3) + gradIn(3);   Jt(3*k+3)  = Jt(3*k+3) + gradOut(3);   
    end

    J 	= AuxData.D * Jt;
    
end
        
    
    

