

%gradient hardware constraints
Gmax        = 25.0;     %[T/m]
Smax        = 150.0;   	%[Ts/m]

verbosity   = 2;
% 0: no output
% 1: objective function options and final results
% 2: objective function options, final results, fmincon iteration outputs

%tuning of optimization
TolFun 	= 1e-8; %Default 1e-6
TolX  	= 1e-8; %Default 1e-6
TolCon  = 1e-8; %Default 1e-6

UseBoundaryConstraints = 1;
%set to 1 to force, that first and last constrol point satisfy the gradient
%boundary constraint G(0) = (Gx_init, Gy_init, Gz_init)
%and                 G(T) = (Gx_end,  Gy_end,  Gz_end)              
Gx_start    = 0.0;
Gy_start    = 0.0;
Gz_start    = 0.0;
Gx_end      = 0.0;
Gy_end      = 0.0;
Gz_end      = 0.0;

% Optimal Gradient Design Solver
%-------------------------------
SolverID_idx    = 4;
ValidSolvers = {'InterPoint_Jacobian',...   % 1: fast and good
            	'InterPoint_Hessian',...    % 2: fast and good
                'MinConF_SPG',...           % 4: slow, very fast trajectories
                'ModifiedShor'};            % 9: slow, fastest trajectories
SolverID = ValidSolvers{ SolverID_idx };


%Some options of the objective function
%--------------------------------------
Options.OnlyContinuousRanges    = 0;
Options.GradSmooth_On           = 1;
Options.GradSmooth_Lambda       = 1e-3; %[ms]
Options.GradSmooth_MaxDistFac   = 2;
Options.verbosity              	= verbosity;
Options.UseMex                  = 0;
Options.SolverID                = SolverID;


% Use some predefined trajectory
%-------------------------------
CPSet = 1;
%  1: Shells Trajectory without constraints
%  2: Shells Trajectory with constraints
%  3: Separated Cross without constraints
%  4: Separated Cross with constraints
%  5: Stack of Spirals without constraints
%  6: Stack of Spirals with constraints

switch CPSet
    case 1
        load('ConPoints_3D_Shells_unconstrained.mat');
    case 2
        load('ConPoints_3D_Shells_constrained.mat');
    case 3
        load('ConPoints_3D_Cross_unconstrained.mat');
  	case 4
        load('ConPoints_3D_Cross_constrained.mat');
    case 5
        load('ConPoints_3D_StackOfSpirals_unconstrained.mat');
  	case 6
        load('ConPoints_3D_StackOfSpirals_constrained.mat');
end

%set boundary constraints
if UseBoundaryConstraints
    R_mat(1,:)    = [Gx_start, Gy_start, Gz_start, 0];
    R_mat(end,:)  = [Gx_end, Gy_end, Gz_end, 0];
end

% Compute Gradient Waveforms
%---------------------------
G0  = []; %start with zero gradient in constrained optimization
T   = PerformOptimization3D( kx_vec, ky_vec, kz_vec, R_mat, Gmax, Smax, G0, Options );

% plot trajectory
PlotTrajectory( T );

% Get hardware-suitable sampling
%-------------------------------
dT  = 10e-6;
T   = CalculateHardwareTrajectory( T, dT );

% plot gradient waveforms using hardware sampling
%(analytic gradients can be plotted by changing 'h' to 'gs')
PlotGradients( T, 'h' );



