function [tlim, gradIn, gradOut, gradIn_In, gradIn_Out, gradOut_In, gradOut_Out] = GetMinimumTime2D( lowBoundsA, uppBoundsA, lowBoundsB, uppBoundsB, OptsVector )

    %this method is used to determine the smallest tlim contained in both
    %ranges A and B, given by n time ranges in lowBoundsA and uppBoundsA
    %and m time ranges in lowBoundsB and uppBoundsB. The method should find
    %a tlim in any case, since each dimension contains a range with upper
    %bound being infinity
    
    %GradLevel
    if      size( lowBoundsA, 2 ) == 1
        GradLevel = 1;
    elseif  size( lowBoundsA, 2 ) == 3
        GradLevel = 2;
    else    %size( lowBoundsA, 2 ) == 7
        GradLevel = 3; 
    end

    %initialize everything
    gradIn      = [];
    gradOut     = [];
    gradIn_In   = []; 
    gradIn_Out  = []; 
    gradOut_In  = []; 
    gradOut_Out = [];

    %preferences
    % UseWeightedGradient = 1;
    % Lambda              = 1e-3; %[ms]
    % MaxDist             = 2 * Lambda;

    UseWeightedGradient = OptsVector(2);
    Lambda              = OptsVector(3); %[ms]
    MaxDist             = OptsVector(4) * Lambda;
    
 	%predefintion
    InitIdx     = find( [lowBoundsA(end,1),lowBoundsB(end,1)] == max([lowBoundsA(end,1),lowBoundsB(end,1)]), 1, 'first');

    switch InitIdx(1)
        case 1
            tlim        = lowBoundsA(end,1);
            Distance    = tlim - lowBoundsB(end,1);
            
            %set gradients
            if GradLevel == 2
                [gradIn, gradOut] = SetWeightedGradients_Jac( lowBoundsA, lowBoundsB, Distance, MaxDist, Lambda, 'A', 'init', UseWeightedGradient );
            elseif GradLevel == 3
                [gradIn, gradOut, ...
                 gradIn_In, gradIn_Out, ...
                 gradOut_In, gradOut_Out ] = SetWeightedGradients_Hess( lowBoundsA, lowBoundsB, Distance, MaxDist, Lambda, 'A', 'init', UseWeightedGradient );
            end

        otherwise
            tlim        = lowBoundsB(end,1);
            Distance    = tlim - lowBoundsA(end,1);
            
            %set gradients
            if GradLevel == 2
                [gradIn, gradOut] = SetWeightedGradients_Jac( lowBoundsA, lowBoundsB, Distance, MaxDist, Lambda, 'B', 'init', UseWeightedGradient );
            elseif GradLevel == 3
                [gradIn, gradOut, ...
                 gradIn_In, gradIn_Out, ...
                 gradOut_In, gradOut_Out ] = SetWeightedGradients_Hess( lowBoundsA, lowBoundsB, Distance, MaxDist, Lambda, 'B', 'init', UseWeightedGradient );
            end
    end

    while(1)
        
        %get largest of lowest bounds in A and B
        CurLim      = double( find(max([lowBoundsA(1),lowBoundsB(1)]) == [lowBoundsA(1),lowBoundsB(1)], 1) );
        
        switch CurLim(1)
            case 1
                %check, if lowest Bound of A is in B
                % YES: return
                %  NO: remove first range of B
                Idx = find( any( lowBoundsA(1,1) <= uppBoundsB(:,1) & lowBoundsA(1,1) >= lowBoundsB(:,1) ),1,'first' );
                if Idx ~= 1
                   error('WTF!') 
                end
                
                if ~isempty( Idx )
                    
                    tlim        = lowBoundsA(1);
                    Distance    = tlim - lowBoundsB(Idx,1);

                    %set gradients
                    if GradLevel == 2
                        [gradIn, gradOut] = SetWeightedGradients_Jac( lowBoundsA, lowBoundsB, Distance, MaxDist, Lambda, 'A', 'inter', UseWeightedGradient );
                    elseif GradLevel == 3
                        [gradIn, gradOut, ...
                         gradIn_In, gradIn_Out, ...
                         gradOut_In, gradOut_Out ] = SetWeightedGradients_Hess( lowBoundsA, lowBoundsB, Distance, MaxDist, Lambda, 'A', 'inter', UseWeightedGradient );
                    end
                    
                    break;
                else
                    lowBoundsB(1,:)   = [];
                    uppBoundsB(1,:)   = []; 
                end
                
            otherwise
                %check, if lowest Bound of B is in A
                % YES: return
                %  NO: remove first range of A
                Idx = find( any( lowBoundsB(1,1) <= uppBoundsA(:,1) & lowBoundsB(1,1) >= lowBoundsA(:,1) ),1,'first' );
                if Idx ~= 1
                   error('WTF!') 
                end
                
                if ~isempty( Idx )
                    tlim        = lowBoundsB(1);
                    Distance    = tlim - lowBoundsA(Idx,1);
                    
                    %set gradients
                    if GradLevel == 2
                        [gradIn, gradOut] = SetWeightedGradients_Jac( lowBoundsA, lowBoundsB, Distance, MaxDist, Lambda, 'B', 'inter', UseWeightedGradient );
                    elseif GradLevel == 3
                        [gradIn, gradOut, ...
                         gradIn_In, gradIn_Out, ...
                         gradOut_In, gradOut_Out ] = SetWeightedGradients_Hess( lowBoundsA, lowBoundsB, Distance, MaxDist, Lambda, 'B', 'inter', UseWeightedGradient );
                    end
                    
                    break;
                else
                    lowBoundsA(1,:)   = [];
                    uppBoundsA(1,:)   = [];
                end
      
        end

        if isempty(lowBoundsA) || isempty(uppBoundsA) || ...
           isempty(lowBoundsB) || isempty(uppBoundsB)
            break;
        end

    end 
    
end

%Compute weighted Jacobian
function [gradIn, gradOut] = SetWeightedGradients_Jac( lowBoundsA, lowBoundsB, Distance, MaxDist, Lambda, dim, state, UseWeightedGradient )

    % entries at Idx are kept as original, other entries are weighted

    switch state
        case 'init'
            BndAIdx = size( lowBoundsA, 1 );
            BndBIdx = size( lowBoundsB, 1 );
        case 'inter'
            BndAIdx = 1;
            BndBIdx = 1;
    end
    
    %set gradients
    if UseWeightedGradient && Distance < MaxDist
        %use weighted gradient
        switch dim
            case 'A'
                gradIn  = [lowBoundsA(BndAIdx,2), exp(-Distance/Lambda) * lowBoundsB(BndBIdx,2)];
                gradOut = [lowBoundsA(BndAIdx,3), exp(-Distance/Lambda) * lowBoundsB(BndBIdx,3)];
            otherwise % 'B'
                gradIn  = [exp(-Distance/Lambda) * lowBoundsA(BndAIdx,2), lowBoundsB(BndBIdx,2)];
                gradOut = [exp(-Distance/Lambda) * lowBoundsA(BndAIdx,3), lowBoundsB(BndBIdx,3)];
        end
    else
        %use original unweighed gradient
        switch dim
            case 'A'
            gradIn	= [lowBoundsA(BndAIdx,2), 0.0];
            gradOut = [lowBoundsA(BndAIdx,3), 0.0];
        otherwise % 'B'
            gradIn	= [0.0, lowBoundsB(BndBIdx,2)];
            gradOut = [0.0, lowBoundsB(BndBIdx,3)];
        end
    end
end

%Compute weighted Jacobian
function [gradIn, gradOut, gradIn_In, gradIn_Out, gradOut_In, gradOut_Out] = SetWeightedGradients_Hess( lowBoundsA, lowBoundsB, Distance, MaxDist, Lambda, dim, state, UseWeightedGradient )

    % entries at Idx are kept as original, other entries are weighted

    switch state
        case 'init'
            BndAIdx = size( lowBoundsA, 1 );
            BndBIdx = size( lowBoundsB, 1 );
        case 'inter'
            BndAIdx = 1;
            BndBIdx = 1;
    end
    
    %set gradients
    if UseWeightedGradient && Distance < MaxDist
        %use weighted gradient
        switch dim
            case 'A'
                gradIn      = [lowBoundsA(BndAIdx,2), exp(-Distance/Lambda) * lowBoundsB(BndBIdx,2)];
                gradOut     = [lowBoundsA(BndAIdx,3), exp(-Distance/Lambda) * lowBoundsB(BndBIdx,3)];
                gradIn_In   = [lowBoundsA(BndAIdx,4), exp(-Distance/Lambda) * lowBoundsB(BndBIdx,4)];
                gradIn_Out  = [lowBoundsA(BndAIdx,5), exp(-Distance/Lambda) * lowBoundsB(BndBIdx,5)];
                gradOut_In  = [lowBoundsA(BndAIdx,6), exp(-Distance/Lambda) * lowBoundsB(BndBIdx,6)];
                gradOut_Out = [lowBoundsA(BndAIdx,7), exp(-Distance/Lambda) * lowBoundsB(BndBIdx,7)];
            otherwise % 'B'
                gradIn      = [exp(-Distance/Lambda) * lowBoundsA(BndAIdx,2), lowBoundsB(BndBIdx,2)];
                gradOut     = [exp(-Distance/Lambda) * lowBoundsA(BndAIdx,3), lowBoundsB(BndBIdx,3)];
                gradIn_In   = [exp(-Distance/Lambda) * lowBoundsA(BndAIdx,4), lowBoundsB(BndBIdx,4)];
                gradIn_Out  = [exp(-Distance/Lambda) * lowBoundsA(BndAIdx,5), lowBoundsB(BndBIdx,5)];
                gradOut_In  = [exp(-Distance/Lambda) * lowBoundsA(BndAIdx,6), lowBoundsB(BndBIdx,6)];
                gradOut_Out = [exp(-Distance/Lambda) * lowBoundsA(BndAIdx,7), lowBoundsB(BndBIdx,7)];
        end
    else
        %use original unweighed gradient
        switch dim
            case 'A'
            gradIn      = [lowBoundsA(BndAIdx,2), 0.0];
            gradOut     = [lowBoundsA(BndAIdx,3), 0.0];
            gradIn_In	= [lowBoundsA(BndAIdx,4), 0.0];
            gradIn_Out  = [lowBoundsA(BndAIdx,5), 0.0];
            gradOut_In	= [lowBoundsA(BndAIdx,6), 0.0];
            gradOut_Out = [lowBoundsA(BndAIdx,7), 0.0];
        otherwise % 'B'
            gradIn      = [0.0, lowBoundsB(BndBIdx,2)];
            gradOut     = [0.0, lowBoundsB(BndBIdx,3)];
            gradIn_In	= [0.0, lowBoundsB(BndBIdx,4)];
            gradIn_Out  = [0.0, lowBoundsB(BndBIdx,5)];
            gradOut_In	= [0.0, lowBoundsB(BndBIdx,6)];
            gradOut_Out = [0.0, lowBoundsB(BndBIdx,7)];
        end
    end
end