function [ Hx, Hy, Hz ] = PlotGradients( T, ID, OpenFigure )

    if nargin < 3
        OpenFigure = 1;
    end
    
    if OpenFigure
        figure
        hold on
        box on
        grid on 
    end
    
    if nargin < 2
       ID = 'g';
    else
       if ~any( cellfun( @(s) strcmpi( s, ID ), {'g','s','gs','gc','h','p'} ) )
%           warning('Identifier ''%s'' undefined, only ''g'',''s'',''gs'', ''gc'', or ''h/p'' are allowed. Using default ''g''!',ID) 
       end
    end
 
    
    %figure
    switch ID
        
        case 'g'
            %Gradient Magnitude Only
            [Hx,Hy,Hz,C,ah1] = PlotWaveforms( T.Ana.tx, T.Ana.ty, T.Ana.tz, T.Ana.Gx, T.Ana.Gy, T.Ana.Gz, T.t, T.Gmax, 'plot' );
            xlabel('Time t [ms]','Fontsize',12,'Fontweight','normal')
            ylabel('Gradient Magnitude G(t) [mT/m]','Fontsize',12,'Fontweight','normal')
            LegHandle = legend([Hx,Hy,Hz,C],'G_x(t)','G_y(t)','G_z(t)','G_{max}');
            set(LegHandle,'orientation','horizontal')
            
            xlim([min(T.t), max(T.t)])
            ylim([-1.1 * T.Gmax, +1.1 * T.Gmax ])
            
        case 's'
            %Gradient SlewRate Only
            [Hx,Hy,Hz,C,ah1] = PlotWaveforms( T.Ana.tx, T.Ana.ty, T.Ana.tz, T.Ana.Sx, T.Ana.Sy, T.Ana.Sz, T.t, T.Smax, 'stairs'  );
            xlabel('Time t [ms]','Fontsize',12,'Fontweight','normal')
            ylabel('Gradient Slew Rate G(t) [T/m/s]','Fontsize',12,'Fontweight','normal')
            LegHandle = legend([Hx,Hy,Hz,C],'S_x(t)','S_y(t)','S_z(t)','S_{max}');
            set(LegHandle,'orientation','horizontal')
            
            xlim([min(T.t), max(T.t)])
            ylim([-1.1 * T.Smax, +1.1 * T.Smax ])
            
    	case 'gs'
            %Gradient Magnitude and SlewRate
            subplot(4,10,1:20)
            [Hx,Hy,Hz,C,ah1] = PlotWaveforms( T.Ana.tx, T.Ana.ty, T.Ana.tz, T.Ana.Gx, T.Ana.Gy, T.Ana.Gz, T.t, T.Gmax, 'plot' );
            xlabel('Time t [ms]','Fontsize',12,'Fontweight','normal')
            ylabel('Gradient Magnitude G(t) [mT/m]','Fontsize',12,'Fontweight','normal')
            LegHandle = legend([Hx,Hy,Hz,C],'G_x(t)','G_y(t)','G_z(t)','G_{max}');
            set(LegHandle,'orientation','horizontal')
            
            xlim([min(T.t), max(T.t)])
            ylim([-1.1 * T.Gmax, +1.1 * T.Gmax ])
            
            subplot(4,10,21:40)
            [Hx,Hy,Hz,C,ah3] = PlotWaveforms( T.Ana.tx, T.Ana.ty, T.Ana.tz, T.Ana.Sx, T.Ana.Sy, T.Ana.Sz, T.t, T.Smax, 'stairs'  );
            xlabel('Time t [ms]','Fontsize',12,'Fontweight','normal')
            ylabel('Gradient Slew Rate G(t) [T/m/s]','Fontsize',12,'Fontweight','normal')
            LegHandle = legend([Hx,Hy,Hz,C],'S_x(t)','S_y(t)','S_z(t)','S_{max}');
            set(LegHandle,'orientation','horizontal')
            
            xlim([min(T.t), max(T.t)])
            ylim([-1.1 * T.Smax, +1.1 * T.Smax ])
            
            linkaxes([ah1, ah3,],'x')
            
        case 'gc'
            %Gradient Magnitude and Constraint Behaviour
            subplot(4,10,1:20)
            [Hx,Hy,Hz,C,ah1] = PlotWaveforms( T.Ana.tx, T.Ana.ty, T.Ana.tz, T.Ana.Gx, T.Ana.Gy, T.Ana.Gz, T.t, T.Gmax, 'plot' );
            xlabel('Time t [ms]','Fontsize',12,'Fontweight','normal')
            ylabel('Gradient Magnitude G(t) [mT/m]','Fontsize',12,'Fontweight','normal')
            LegHandle = legend([Hx,Hy,Hz,C],'G_x(t)','G_y(t)','G_z(t)','G_{max}');
            set(LegHandle,'orientation','horizontal')
            
            xlim([min(T.t), max(T.t)])
            ylim([-1.1 * T.Gmax, +1.1 * T.Gmax ])
            
            subplot(4,10,21:40)
            [Magn, Slew, ah3] = PlotConstraints( T );
            xlabel('Time t [ms]','Fontsize',12,'Fontweight','normal')
            ylabel('Active Constraints','Fontsize',12,'Fontweight','normal')
            LegHandle = legend([Magn,Slew],'Magnitude Constraint','Slew Rate Constraint');
            set(LegHandle,'orientation','horizontal')
            
            xlim([min(T.t), max(T.t)])
            ylim([-1,4])
            
            linkaxes([ah1, ah3],'x')
            
        case {'h','p'}
            
            %first check, if hardware gradients exist
            if  isempty( T.Pulse.Gx ) || isempty( T.Pulse.Gy ) || isempty( T.Pulse.Gz ) || ...
                isempty( T.Pulse.Sx ) || isempty( T.Pulse.Sy ) || isempty( T.Pulse.Sz ) || ...
                isempty( T.Pulse.kx ) || isempty( T.Pulse.ky ) || isempty( T.Pulse.kz ) || ...
                isempty( T.Pulse.t )
                    error('No hardware trajectory has beend efined T-struct!')
            end
            
            %Gradient Magnitude and SlewRate
            subplot(4,10,1:20)
            [Hx,Hy,Hz,C,ah1] = PlotWaveforms( T.Pulse.t, T.Pulse.t, T.Pulse.t, T.Pulse.Gx, T.Pulse.Gy, T.Pulse.Gz, [], T.Gmax/1e+3, 'stairs' );
            xlabel('Time t [s]','Fontsize',12,'Fontweight','normal')
            ylabel('Gradient Magnitude G(t) [T/m]','Fontsize',12,'Fontweight','normal')
            LegHandle = legend([Hx,Hy,Hz,C],'G_x(t)','G_y(t)','G_z(t)','G_{max}');
            set(LegHandle,'orientation','horizontal')
            
            xlim([min(T.Pulse.t), max(T.Pulse.t)])
            ylim([-1.1 * T.Gmax / 1e+3, +1.1 * T.Gmax / 1e+3 ])
            
            subplot(4,10,21:40)
            [Hx,Hy,Hz,C,ah3] = PlotWaveforms( T.Pulse.t, T.Pulse.t, T.Pulse.t, T.Pulse.Sx, T.Pulse.Sy, T.Pulse.Sz, [], T.Smax, 'stairs' );
            xlabel('Time t [s]','Fontsize',12,'Fontweight','normal')
            ylabel('Gradient Slew Rate S(t) [T/m/s]','Fontsize',12,'Fontweight','normal')
            LegHandle = legend([Hx,Hy,Hz,C],'S_x(t)','S_y(t)','S_z(t)','S_{max}');
            set(LegHandle,'orientation','horizontal')
            
            xlim([min(T.Pulse.t), max(T.Pulse.t)])
            ylim([-1.1 * T.Smax, +1.1 * T.Smax ])
            
            linkaxes([ah1, ah3],'x')
            
    	case {'hg','pg'}
            
            %first check, if hardware gradients exist
            if  isempty( T.Pulse.Gx ) || isempty( T.Pulse.Gy ) || isempty( T.Pulse.Gz ) || ...
                isempty( T.Pulse.Sx ) || isempty( T.Pulse.Sy ) || isempty( T.Pulse.Sz ) || ...
                isempty( T.Pulse.kx ) || isempty( T.Pulse.ky ) || isempty( T.Pulse.kz ) || ...
                isempty( T.Pulse.t )
                    error('No hardware trajectory has beend efined T-struct!')
            end
            
            %Gradient Magnitude and SlewRate
            [Hx,Hy,Hz,C,~] = PlotWaveforms( T.Pulse.t, T.Pulse.t, T.Pulse.t, T.Pulse.Gx, T.Pulse.Gy, T.Pulse.Gz, [], T.Gmax/1e+3, 'stairs' );
            xlabel('Time t [s]','Fontsize',12,'Fontweight','normal')
            ylabel('Gradient Magnitude G(t) [T/m]','Fontsize',12,'Fontweight','normal')
            LegHandle = legend([Hx,Hy,Hz,C],'G_x(t)','G_y(t)','G_z(t)','G_{max}');
            set(LegHandle,'orientation','horizontal')
            
            xlim([min(T.Pulse.t), max(T.Pulse.t)])
            ylim([-1.1 * T.Gmax / 1e+3, +1.1 * T.Gmax / 1e+3 ])
        
    	case {'hs','ps'}
            
            %first check, if hardware gradients exist
            if  isempty( T.Pulse.Gx ) || isempty( T.Pulse.Gy ) || isempty( T.Pulse.Gz ) || ...
                isempty( T.Pulse.Sx ) || isempty( T.Pulse.Sy ) || isempty( T.Pulse.Sz ) || ...
                isempty( T.Pulse.kx ) || isempty( T.Pulse.ky ) || isempty( T.Pulse.kz ) || ...
                isempty( T.Pulse.t )
                    error('No hardware trajectory has beend efined T-struct!')
            end

            [Hx,Hy,Hz,C,~] = PlotWaveforms( T.Pulse.t, T.Pulse.t, T.Pulse.t, T.Pulse.Sx, T.Pulse.Sy, T.Pulse.Sz, [], T.Smax, 'stairs' );
            xlabel('Time t [s]','Fontsize',12,'Fontweight','normal')
            ylabel('Gradient Slew Rate S(t) [T/m/s]','Fontsize',12,'Fontweight','normal')
            LegHandle = legend([Hx,Hy,Hz,C],'S_x(t)','S_y(t)','S_z(t)','S_{max}');
            set(LegHandle,'orientation','horizontal')
            
            xlim([min(T.Pulse.t), max(T.Pulse.t)])
            ylim([-1.1 * T.Smax, +1.1 * T.Smax ])
  
    end
        
    if OpenFigure
        set(gcf,'OuterPosition',[205.000000, 104.000000, 1208.000000, 859.000000]);
        set(gcf,'Position',[213.000000, 112.000000, 1192.000000, 767.000000]);
        movegui(gcf,'center')
    end
    
end

function [Hx,Hy,Hz,C,ah1] = PlotWaveforms( tx, ty, tz, fx, fy, fz, Interp, Constraint, FunType )

    Linewidth = 1.0;

    %prepare axis
    ah1 = gca;
    box on
    grid on
    hold on
    
    %plot constraints
        line(   [min([tx(1),ty(1)]), max([tx(end),ty(end)])], ...
            [+Constraint, +Constraint],'Linestyle','--','Linewidth',2,'Color',[0 0.7 0]);
    C = line(   [min([tx(1),ty(1)]), max([tx(end),ty(end)])], ...
            [-Constraint, -Constraint],'Linestyle','--','Linewidth',2,'Color',[0 0.7 0]);    
    
	%this method plots the analytic gradients
    switch FunType
        case 'plot'
            Hx = plot( tx, fx, 'Linewidth',Linewidth,'Color','red');
            Hy = plot( ty, fy, 'Linewidth',Linewidth,'Color','blue');
            Hz = plot( tz, fz, 'Linewidth',Linewidth,'Color',[0 0.7 0]);
            
            %plot patch
            AlphaVal = 0.35;
            phx = patch([tx(1),tx,tx(end)],[0.0,fx,0.0],'red','Edgecolor','none');
            alpha(phx,AlphaVal);
            phy = patch([ty(1),ty,ty(end)],[0.0,fy,0.0],'blue','Edgecolor','none');
            alpha(phy,AlphaVal);
            phz = patch([tz(1),tz,tz(end)],[0.0,fz,0.0],[0 0.7 0],'Edgecolor','none');
            alpha(phz,AlphaVal);
            
        case 'stairs'
            Hx = stairs( tx, fx, 'Linewidth',Linewidth,'Color','red');
            Hy = stairs( ty, fy, 'Linewidth',Linewidth,'Color','blue');
            Hz = stairs( tz, fz, 'Linewidth',Linewidth,'Color',[0 0.7 0]);
            
            %create some auxilary vectors to allow for nice plotting of the
            %Smax waveforms
            
            tx_temp = reshape([tx; tx],1,[]);
            tx_temp = tx_temp( 2 : end-1 );
            fx_temp = reshape([fx; fx],1,[]);
            fx_temp = fx_temp( 1 : end-2 );
            
            ty_temp = reshape([ty; ty],1,[]);
            ty_temp = ty_temp( 2 : end-1 );
            fy_temp = reshape([fy; fy],1,[]);
            fy_temp = fy_temp( 1 : end-2 );
            
            tz_temp = reshape([tz; tz],1,[]);
            tz_temp = tz_temp( 2 : end-1 );
            fz_temp = reshape([fz; fz],1,[]);
            fz_temp = fz_temp( 1 : end-2 );
            
            
            %plot patch
            AlphaVal = 0.35;
            phx = patch([tx_temp(1),tx_temp,tx_temp(end)],[0.0,fx_temp,0.0],'red','Edgecolor','none');
            alpha(phx,AlphaVal);
            phy = patch([ty_temp(1),ty_temp,ty_temp(end)],[0.0,fy_temp,0.0],'blue','Edgecolor','none');
            alpha(phy,AlphaVal);
            phz = patch([tz_temp(1),tz_temp,tz_temp(end)],[0.0,fz_temp,0.0],[0 0.7 0],'Edgecolor','none');
            alpha(phz,AlphaVal);
            
    end

    %this method plots the times of the interpolation as vertical lines
%     for k = 1 : numel(Interp)
%        line([Interp(k), Interp(k)], [-Constraint, +Constraint], ...
%           'Linestyle','-','Linewidth',1,'Color',[1 0.55 0]); 
%     end

end

function [Magn, Slew, ah1] = PlotConstraints( T )

    %prepare axis
    ah1 = gca;
    box on
    grid on
    hold on
    
    % first of all, determine intervals at which the Magnitude OR SlewRate
    % is the current active constraint
    T = CalculateSampledTrajectory( T );
    ActiveConstraint_SlewRate   = max( abs([T.Sam.Sx(:), T.Sam.Sy(:), T.Sam.Sz(:)]), [], 2 ) >= 0.99 * T.Smax;
    ActiveConstraint_Magnitude  = max( abs([T.Sam.Gx(:), T.Sam.Gy(:), T.Sam.Gz(:)]), [], 2 ) >= 0.99 * T.Gmax;
    
    %perform morphological closing on both vectors
    ActiveConstraint_SlewRate   = imclose( ActiveConstraint_SlewRate, [1,1]' );
    ActiveConstraint_Magnitude  = imclose( ActiveConstraint_Magnitude, [1,1]' );
    Components_SlewRate         = bwconncomp( ActiveConstraint_SlewRate, 4 );
    Components_Magnitude        = bwconncomp( ActiveConstraint_Magnitude, 4 );
    
    MagnBounds  = [1.5, 3.0];
    SlewBounds  = [0.0, 1.5];
    
    %plot patches
    for idx_slew = 1 : Components_SlewRate.NumObjects
        Slew = rectangle('Position',[T.Sam.t(Components_SlewRate.PixelIdxList{idx_slew}(1)),SlewBounds(1), ...
                                     T.Sam.t(Components_SlewRate.PixelIdxList{idx_slew}(end)) - T.Sam.t(Components_SlewRate.PixelIdxList{idx_slew}(1)),diff(SlewBounds)], ...
                                     'FaceColor','black','EdgeColor','none');
    end
    for idx_magn = 1 : Components_Magnitude.NumObjects
        Magn = rectangle('Position',[T.Sam.t(Components_Magnitude.PixelIdxList{idx_magn}(1)),MagnBounds(1), ...
                                     T.Sam.t(Components_Magnitude.PixelIdxList{idx_magn}(end)) - T.Sam.t(Components_Magnitude.PixelIdxList{idx_magn}(1)),diff(MagnBounds)], ...
                                     'FaceColor','red','EdgeColor','none');
    end
    
    Slew = plot(nan,nan,'s','markeredgecolor',get(Slew,'edgecolor'),'markerfacecolor',get(Slew,'facecolor'));
    Magn = plot(nan,nan,'s','markeredgecolor',get(Magn,'edgecolor'),'markerfacecolor',get(Magn,'facecolor'));
    
    
    xlim([0.0, T.Sam.t(end)])
    ylim([-1.0, 4.0])
    set(gca,'YTick',[])
    
end

