function [Gn, tn] = RemoveAnaticalSolutionDuplicates( Gn, tn, delta_t, delta_G, verbose )

    %this method is used to remove duplicate entrys in the analytic
    %solution

    if nargin < 5
        verbose  = 0;
    end
    
    %we only need the analytic solution in both dimensions
    if isempty( Gn ) || isempty( tn )
        error('No analytic solution available!')
    end
    
    if numel( Gn ) ~= numel( tn )
        error('Both Gn and tn need to have same length!')
    end

    %find duplicates in Gn
    tDupInds    = find( abs(tn(1:end-1) - tn(2:end)) < delta_t );
    
    %check, if t duplicates indicate corresponding G duplicates
    GDupBool    = double( abs( Gn(tDupInds) - Gn(tDupInds+1) ) < delta_G );
    
    if ~isempty( GDupBool )
        if min(GDupBool) == 0 && verbose > 0
            warning('WarnTests:convertTest','Some of the duplicates in tn do NOT correspond to duplicates in the gradient vector. They will be removed anyway!') 
        end
    end

    %perform some output to command
    if numel(tDupInds) > 0 && verbose > 0
       fprintf('Duplicates found: %i/%i entry removed!\n',numel(tDupInds),numel(Gn)) 
    end
    
  	tn( tDupInds ) = [];
    tn( 1 )        = 0.0;
    Gn( tDupInds ) = [];