function [ options, B0, B1 ] = SetPulseOptions3D( options )

    % This method prepares some options that are needed during the shape
    % optimization

    %channel
    %-------
  	options.nchannel = size( options.b1maps, 4 );
   
    % if matrix sizes are not integer of undersampling, crop matrices
    cropx_1 = floor( mod( size( options.b1maps, 1 ), options.undersam_x )/2 );
    cropx_2 =  ceil( mod( size( options.b1maps, 1 ), options.undersam_x )/2 );
    cropy_1 = floor( mod( size( options.b1maps, 2 ), options.undersam_y )/2 );
    cropy_2 =  ceil( mod( size( options.b1maps, 2 ), options.undersam_y )/2 );
    cropz_1 = floor( mod( size( options.b1maps, 3 ), options.undersam_z )/2 );
    cropz_2 =  ceil( mod( size( options.b1maps, 3 ), options.undersam_z )/2 );

    %if ratio of number of pixels and undersampling factor is not an
    %integer number, crop the maps accordingly
    if any( [ cropx_1, cropx_2, cropy_1, cropy_2, cropz_1, cropz_2 ] ) > 0
        fprintf('Undersampling factors do not match: matrices will be cropped (X: %i/%i, Y: %i/%i, Z: %i/%i)\n',cropx_1,cropx_2,cropy_1,cropy_2,cropz_1,cropz_2)
        options.b1maps          = options.b1maps( cropx_1 + 1 : end - cropx_2, cropy_1 + 1 : end - cropy_2, cropz_1 + 1 : end - cropz_2, : );
        options.b0map           = options.b0map(  cropx_1 + 1 : end - cropx_2, cropy_1 + 1 : end - cropy_2, cropz_1 + 1 : end - cropz_2 );
        options.roi             = options.roi(    cropx_1 + 1 : end - cropx_2, cropy_1 + 1 : end - cropy_2, cropz_1 + 1 : end - cropz_2 );
        options.xpos_vec        = options.xpos_vec( cropx_1 + 1 : end - cropx_2 );
        options.ypos_vec        = options.ypos_vec( cropy_1 + 1 : end - cropy_2 );
        options.zpos_vec        = options.zpos_vec( cropz_1 + 1 : end - cropz_2 );

        if exist('options.target_mat','var')
            options.target_mat = options.target_mat( cropx_1 + 1 : end - cropx_2, cropy_1 + 1 : end - cropy_2, cropz_1 + 1 : end - cropz_2, : );
        end
    end

    options.nx = size( options.b1maps, 1 ) / options.undersam_x;
    options.ny = size( options.b1maps, 2 ) / options.undersam_y;
    options.nz = size( options.b1maps, 3 ) / options.undersam_z;

    if options.undersam_x > 1 || options.undersam_y > 1 || options.undersam_z > 1
        %average pixel locations, make sure they are vectors
        options.xpos_vec   = reshape( BlockMean( options.xpos_vec, options.undersam_x ), [], 1 );
        options.ypos_vec   = reshape( BlockMean( options.ypos_vec, options.undersam_y ), [], 1 );
        options.zpos_vec   = reshape( BlockMean( options.zpos_vec, options.undersam_z ), [], 1 );

        %average b1
        b1maps_full     = options.b1maps;
        options.b1maps  = nan( options.nx, options.ny, options.nz, options.nchannel );
        for chn = 1 : size(options.b1maps, 4)
            fprintf('Averaging B1 map to [%i, %i, %i] (%i/%i)... ',options.nx, options.ny, options.nz, chn, options.nchannel)
            options.b1maps(:,:,:,chn) = BlockMean( b1maps_full(:,:,:,chn), options.undersam_x, options.undersam_y, options.undersam_z );
            fprintf(' done.\n')
        end
        clear b1maps_full

        %average b0
        fprintf('Averaging B0 map to [%i, %i, %i] ... ',options.nx, options.ny, options.nz )
        options.b0map = BlockMean( options.b0map, options.undersam_x, options.undersam_y, options.undersam_z );
        fprintf(' done.\n')

        %average roi
        fprintf('Averaging Mask to [%i, %i, %i] ... ',options.nx, options.ny, options.nz )
        options.roi = floor( BlockMean( options.roi, options.undersam_x, options.undersam_y, options.undersam_z ) );
        fprintf(' done.\n')

        %average target FA
        fprintf('Averaging target pattern to [%i, %i, %i] ... ',options.nx, options.ny, options.nz )
        options.targetflip_mat = BlockMean( options.targetflip_mat, options.undersam_x, options.undersam_y, options.undersam_z );
        fprintf(' done.\n')

    end

    % get pixel sizes after cropping and downsampling
    options.PixSiz_x  	= abs(options.xpos_vec(2) - options.xpos_vec(1));
    options.PixSiz_y  	= abs(options.ypos_vec(2) - options.ypos_vec(1));
    options.PixSiz_z  	= abs(options.zpos_vec(2) - options.zpos_vec(1));
        
    %set mask
    options.roi = double( options.roi );
    
    %set indices and positions of nonzeropixels
    Idx     = find( options.roi == 1 );
    [ options.xidx_all, options.yidx_all, options.zidx_all ] = ind2sub( [options.nx, options.ny, options.nz], Idx );

    [X,Y,Z]	= ndgrid( options.xpos_vec, options.ypos_vec, options.zpos_vec );
    
    options.xpos_all  = X( Idx );
    options.ypos_all  = Y( Idx );
    options.zpos_all  = Z( Idx );

    %set number of mask pixel
    options.npixel	= size(Idx, 1);
    
    %get B0 map
    B0  = options.b0map;

    %load B1 maps
    B1 	= cell(options.nchannel,1);
    for chn = 1 : options.nchannel
        B1{chn} = squeeze( options.b1maps( :,:,:,chn ) );
    end
    
    %some command output
    fprintf('\nSetup Summary\n')
 	fprintf('-------------\n')
    fprintf('            Channels: %i\n',options.nchannel)
    fprintf('              Matrix: %i x %i x %i [px] \n',options.nx,options.ny,options.nz)
    fprintf('          Pixel Size: %.2f x %.2f x %.2f [mm] \n',1000*options.PixSiz_x,1000*options.PixSiz_y,1000*options.PixSiz_z)
    fprintf('    Number of Pixels: %i\n',options.npixel)
    
end
